'-----------------------------------------------------------
'         MCUs in Parallel Programmer
'
'         10-1-2008
'      
'
'   
'
' This program is tested to be compiled with Visual Basic 2005 Express.
' This program is tested to run in a Pentium IV at 3.0 GHZ.
'
' The program needs the DLL inpout32.dll
' The program assumes that the parallel poot is LPT1 that 
' responses at I/O addresses:
'   888(dec) for output D7 .. D0
'   889(dec) for input Busy(pin 11)-nACK(pin 10)-X-X-X-X-X-X
'   (invertion is performed only in bit 7 of the byte read from 889)
'   890(dec) for output X-X-X-X-nSelect(pin 17)-X-AutoLF(pin 14)-nStrobe(pin 1)
'   (inversion is performed in bits 0, 1, 3 of the byte written to port address 890).
'
' The Parallel Port configuration must be:
'
' Busy (pin 11):   Connected to GND throught 100 Ohm (inv)
' nStrobe (pin 1): MCLR
' AutoLF (pin 14): PGC
' nSelect(pin 17): PGD out
' nAck(pin 10):    PGD in
'
' The program assumes that the parallel port cable is not removed while the program runs.
'
' The measurements have shown that there is no need for the <waitpc> routine,
' as it takes at least 13usec for the out command to execute!!
'----------------------------------------------------------




Imports System.Threading
Imports System.IO


Public Class Form1
    Public Declare Function Inp Lib "inpout32.dll" _
    Alias "Inp32" (ByVal PortAddress As Integer) As Integer
    Public Declare Sub Out Lib "inpout32.dll" _
    Alias "Out32" (ByVal PortAddress As Integer, ByVal Value As Integer)


    Declare Function QueryPerformanceCounter Lib "Kernel32" (ByRef X As Long) As Short
    Declare Function QueryPerformanceFrequency Lib "Kernel32" (ByRef X As Long) As Short

    Dim reset As Integer = 0
    Dim instr(5632) As Long

    Dim fnameIN As String = "."

    Sub waitpc()
        '
        '   This subroutine introduces a delay of 1 usec for the parallel port operations.
        '

        Static flaginit As Integer = 0      'Marks the first entry of the subroutine
        Static flagcounter As Integer       'Marks the existance of performance Counters
        Static wcount As Long
        Dim Ctr1, Ctr2, Freq As Long

        If flaginit = 0 Then
            If QueryPerformanceCounter(Ctr1) Then
                flagcounter = 1
                QueryPerformanceFrequency(Freq)
                wcount = 1 + Freq / 1000000    'count value for at least 1.0 usec (= 1 MHZ)
            Else
                flagcounter = 0
            End If
            flaginit = 1
        End If

        If flagcounter <> 0 Then
            QueryPerformanceCounter(Ctr1)   ' Begin timing.
            Do
                QueryPerformanceCounter(Ctr2)       ' Finish timing.
            Loop Until (Ctr2 - Ctr1) > wcount
        Else
            Thread.Sleep(1)
        End If

    End Sub
    Sub wait100()
        '
        '   This usbroutine just waits for 100 secs to overcome some problems with my PC
        '   (the software or hardware of my PC interacts with the paralel port
        '    when any of the input pis is set to logic '0'. This situations stops
        '    after about 80-90 seconds. This is the reason for the resistor that
        '    connects the "busy" input pin to the ground.)
        '
        '
        Dim time1, time2 As Double

        time1 = Microsoft.VisualBasic.DateAndTime.Timer
        time2 = Microsoft.VisualBasic.DateAndTime.Timer


        Do While (time2 - time1) < 100

            Label1.Text = "waiting for " & Int(100 - (time2 - time1)) & " sec"
            Application.DoEvents()
            time2 = Microsoft.VisualBasic.DateAndTime.Timer
            ' Do other processing while waiting for 100 seconds to elapse.
        Loop


    End Sub

    Sub ICSP_zero()
        '
        ' this subroutine just sets PGC=0, PGD=0
        ' assumes global variable reset
        ' <reset> = 0 ==> MCLR = 0, <reset> = 1 ==> MCLR = 1
        ' NOTE: All signals are driven through inverters.
        '
        Dim x As Integer

        x = &H0                   '  (0000) 0(0)00: PGD=0, PGC=0, MCLR=0
        If reset = 1 Then x = &H1 '  (0000) 0(0)01: PGD=0, PGC=0, MCLR=1

        Out(890, x)


    End Sub
    Function ICSP_IO(ByVal PGD As Integer) As Integer
        '
        '   PGC, PGD have initial value = 0.
        '   PGC maps to 'AutoLF'
        '   PGD-MOSI maps to 'nSelect'
        '
        '   PGD-MISO maps to nACK.
        '   Returned value is 0 or 1
        '
        '   ATTENTION: it is assumed that all 4 signals are driven through 
        '   hardware inverters!
        '   So, the signals monitored on port's pins are inverted!
        '
        '   Assumes that PGC is in state 0.
        '   assumes reset is a global variable.
        '

        Dim x, y As Integer

        x = &H0                   '  (0000) 0(0)00: PGD=0, PGC=0, MCLR=0
        If reset = 1 Then x = &H1 '  (0000) 0(0)01: PGD=0, PGC=0, MCLR=1
        ' PGC is 0, set PGD
        If PGD = 1 Then x = x Or &H8 '(0000) 1(0)00: PGD=1

        Out(890, x)                 ' Set PGC 0, set PGD
        Out(890, x Or &H2)          ' Set PGC 1   (0000) 0(0)10==> PGC=1
        Out(890, x)                 ' Set PGC 0

        x = Inp(889) And &H40       'read port, mask nAck bit
        y = x >> 6                  'Set data to LSBit
        y = (Not y) And &H1         'invert LSBit



        Return (y)

    End Function
    Sub ICSP_send_word(ByVal x As Long, ByVal n As Integer)
        '
        ' Sends <n> bits of <x> through the ICSP_IO function.
        '
        '
        Dim i As Integer
        Dim test As Long
        Dim data As Integer


        For i = 1 To n
            test = x And &H1
            data = test
            ICSP_IO(test)
            x = x >> 1
        Next i

        ' set lines to 0
        ICSP_zero()


    End Sub
    Sub ICSP_send_key(ByVal x As Long)
        '
        ' Sends 32 bits of <x> through the ICSP_IO function, MSBit first !!!!!
        'This subroutine is used only for sending the key to enter ICSP mode !!
        '
        Dim i As Integer
        Dim temp As Long
        Dim data As Integer



        For i = 31 To 0 Step -1
            temp = (x >> i) And &H1
            data = temp
            ICSP_IO(data)
        Next i

        ' set lines to 0
        ICSP_zero()

    End Sub
   
    Sub ICSP_send_opcode(ByVal x As Long)
        '
        ' Sends the command 0000 and the 24-bit opcode stored in <x>
        '
        '
        Dim i As Integer
        Dim test As Long
        'Dim data As Integer
        Dim base, d As Integer

        ' Send SIX command 0000
        'ICSP_IO(0)
        'ICSP_IO(0)
        'ICSP_IO(0)
        'ICSP_IO(0)


        'For i = 1 To 24
        ' test = x And &H1
        ' data = test
        ' ICSP_IO(test)
        ' x = x >> 1
        ' Next i

        ' set lines to 0
        'ICSP_zero()



        base = &H0                   '  (0000) 0(0)00: PGD=0, PGC=0, MCLR=0
        If reset = 1 Then base = &H1 '  (0000) 0(0)01: PGD=0, PGC=0, MCLR=1


        ' Send SIX command 0000

        d = base                   '  (0000) 0(0)00: PGD=0, PGC=0, MCLR=(reset)

        Out(890, d)                 ' Set PGC 0, PGD=0
        Out(890, d Or &H2)          ' Set PGC 1   (0000) 0(0)10==> PGC=1
        Out(890, d)                 ' Set PGC 0
        Out(890, d Or &H2)          ' Set PGC 1   (0000) 0(0)10==> PGC=1
        Out(890, d)                 ' Set PGC 0
        Out(890, d Or &H2)          ' Set PGC 1   (0000) 0(0)10==> PGC=1
        Out(890, d)                 ' Set PGC 0
        Out(890, d Or &H2)          ' Set PGC 1   (0000) 0(0)10==> PGC=1
        Out(890, d)                 ' Set PGC 0



        For i = 1 To 24
            test = x And &H1

            d = base                      '(0000) 0(0)0r: PGD=0, PGC=0, MCLR=(reset)
            If test = 1 Then d = d Or &H8 '(0000) 1(0)0r: PGD=1

            Out(890, d)                 ' Set PGC 0, set PGD
            Out(890, d Or &H2)          ' Set PGC 1   (0000) 0(0)10==> PGC=1
            Out(890, d)                 ' Set PGC 0


            x = x >> 1
        Next i

        ' set lines to 0
        'ICSP_zero()




    End Sub
    
    Function ICSP_receive_word() As Long
        '
        ' Reads the VISI register.
        '
        '
        Dim i As Integer
        Dim x As Long


        ' Send SIX command 0001
        ICSP_IO(1)
        ICSP_IO(0)
        ICSP_IO(0)
        ICSP_IO(0)

        Thread.Sleep(1)

        For i = 1 To 8
            x = ICSP_IO(1)
        Next i



        'x = 0
        For i = 1 To 15
            x = x + (ICSP_IO(1) << i)
        Next i
        ICSP_IO(1)


        Return (x)

    End Function
    Sub ICSP_erase_flash_full()

        Dim x As Long

        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&H2404FA)      'MOV  #0x404F,W10
        ICSP_send_opcode(&H883B0A)      'MOV  W10,NVMCON

        ICSP_send_opcode(&HA8E761)      'BSET  NVMCON, #WR (BIT 15)
        ICSP_send_opcode(&H0)           'NOP

        Do
            ICSP_send_opcode(&H40200)       'GOTO 0x200
            ICSP_send_opcode(&H0)           'NOP
            ICSP_send_opcode(&H207841)      'MOV  #VISI,W1
            ICSP_send_opcode(&H207602)      'MOV  NVMCON,W2
            ICSP_send_opcode(&H780892)      'MOV  W2,[W1]
            ICSP_send_opcode(&H0)           'NOP

            x = ICSP_receive_word()
            ICSP_send_opcode(0)

        Loop Until (x And &H8000) = 0


        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP


    End Sub
    Sub ICSP_program_config_words()
        Dim x, y As Long

        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&H22BFC7)      'MOV  #0x2BFC,W7

        ICSP_send_opcode(&H24003A)      'MOV  #0x4003,W10
        ICSP_send_opcode(&H883B0A)      'MOV  W10,NVMCON

        ICSP_send_opcode(&H200000)      'MOV  #0x00,W0
        ICSP_send_opcode(&H880190)      'MOV  W0,TABLPAG

        ICSP_send_opcode(&H2FACC6)      'MOV  #0xFACC,W6        (CW2)

        ICSP_send_opcode(&HBB1B86)      'TBLWTL  W6,[W7++]
        ICSP_send_opcode(&H0)           'NOP
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&HA8E761)      'BSET  NVMCON, #WR (BIT 15)
        ICSP_send_opcode(&H0)           'NOP

        Do
            ICSP_send_opcode(&H40200)       'GOTO 0x200
            ICSP_send_opcode(&H0)           'NOP
            ICSP_send_opcode(&H207841)      'MOV  #VISI,W1
            ICSP_send_opcode(&H207602)      'MOV  NVMCON,W2
            ICSP_send_opcode(&H780892)      'MOV  W2,[W1]
            ICSP_send_opcode(&H0)           'NOP

            x = ICSP_receive_word()
            ICSP_send_opcode(0)

        Loop Until (x And &H8000) = 0


        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP




        ICSP_send_opcode(&H200000)      'MOV  #0x00,W0
        ICSP_send_opcode(&H880190)      'MOV  W0,TABLPAG

        ICSP_send_opcode(&H23F7F6)      'MOV  #0x3F7F,W6        (CW1)

        ICSP_send_opcode(&HBB1B86)      'TBLWTL  W6,[W7++]
        ICSP_send_opcode(&H0)           'NOP
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&HA8E761)      'BSET  NVMCON, #WR (BIT 15)
        ICSP_send_opcode(&H0)           'NOP


        Do
            ICSP_send_opcode(&H40200)       'GOTO 0x200
            ICSP_send_opcode(&H0)           'NOP
            ICSP_send_opcode(&H207841)      'MOV  #VISI,W1
            ICSP_send_opcode(&H207602)      'MOV  NVMCON,W2
            ICSP_send_opcode(&H780892)      'MOV  W2,[W1]
            ICSP_send_opcode(&H0)           'NOP

            y = ICSP_receive_word()
            ICSP_send_opcode(0)

        Loop Until (y And &H8000) = 0

        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP



    End Sub
    Sub ICSP_program_config_words_2()
        Dim x, y As Long

        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&H22BFC7)      'MOV  #0x2BFC,W7

        ICSP_send_opcode(&H24003A)      'MOV  #0x4003,W10
        ICSP_send_opcode(&H883B0A)      'MOV  W10,NVMCON

        ICSP_send_opcode(&H200000)      'MOV  #0x00,W0
        ICSP_send_opcode(&H880190)      'MOV  W0,TABLPAG

        ICSP_send_opcode(&H27DCC6)      'MOV  #0x7DCC,W6        (CW2)

        ICSP_send_opcode(&HBB1B86)      'TBLWTL  W6,[W7++]
        ICSP_send_opcode(&H0)           'NOP
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&HA8E761)      'BSET  NVMCON, #WR (BIT 15)
        ICSP_send_opcode(&H0)           'NOP

        Do
            ICSP_send_opcode(&H40200)       'GOTO 0x200
            ICSP_send_opcode(&H0)           'NOP
            ICSP_send_opcode(&H207841)      'MOV  #VISI,W1
            ICSP_send_opcode(&H207602)      'MOV  NVMCON,W2
            ICSP_send_opcode(&H780892)      'MOV  W2,[W1]
            ICSP_send_opcode(&H0)           'NOP

            x = ICSP_receive_word()
            ICSP_send_opcode(0)

        Loop Until (x And &H8000) = 0


        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP




        ICSP_send_opcode(&H200000)      'MOV  #0x00,W0
        ICSP_send_opcode(&H880190)      'MOV  W0,TABLPAG

        ICSP_send_opcode(&H2BF7F6)      'MOV  #0xBF7F,W6        (CW1)

        ICSP_send_opcode(&HBB1B86)      'TBLWTL  W6,[W7++]
        ICSP_send_opcode(&H0)           'NOP
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&HA8E761)      'BSET  NVMCON, #WR (BIT 15)
        ICSP_send_opcode(&H0)           'NOP


        Do
            ICSP_send_opcode(&H40200)       'GOTO 0x200
            ICSP_send_opcode(&H0)           'NOP
            ICSP_send_opcode(&H207841)      'MOV  #VISI,W1
            ICSP_send_opcode(&H207602)      'MOV  NVMCON,W2
            ICSP_send_opcode(&H780892)      'MOV  W2,[W1]
            ICSP_send_opcode(&H0)           'NOP

            y = ICSP_receive_word()
            ICSP_send_opcode(0)

        Loop Until (y And &H8000) = 0

        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP



    End Sub

    Private Sub Button3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button3.Click

        wait100()

    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        reset = 0

        Out(890, &H0)

    End Sub

    Private Sub Button8_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button8.Click
        reset = 1

        Out(890, &H1)

    End Sub

    Sub ICSP_enable()
        '
        ' This routine sends the signals that initiate ICSP.
        ' It manipulates MCLR and sends the ICSP key.
        ' MCLR is left at state '1'
        '

        reset = 0
        ICSP_zero()

        reset = 1
        ICSP_zero()

        reset = 0
        ICSP_zero()

        ICSP_send_key(&H4D434851)

        reset = 1
        ICSP_zero()

    End Sub
    Private Sub Button11_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button11.Click

        Label3.Text = "--"
        ICSP_enable()

    End Sub
    Sub ICSP_sync()
        '
        ' This routine sneds the necessary zero bits for the initial sync after
        ' entering the ICSP mode.
        '

        ICSP_send_word(0, 5)

    End Sub
    Sub ICSP_reset()
        '
        ' This routine sends the initial commands to exit the RESET vector. 
        ' These commands are required for proper operation.
        '

        ICSP_send_opcode(&H40200)
        ICSP_send_opcode(&H40200)
        ICSP_send_opcode(0)

    End Sub
    Private Sub Button15_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button15.Click
        '
        ' This routine sends the first SIX command and the initial commands 
        ' to exit the RESET vector. 
        ' These commands are required for proper operation.
        '
        '

        ICSP_sync()
        ICSP_reset()


    End Sub
    Sub ICSP_readID(ByRef x As Long, ByRef y As Long)
        '
        ' This routine reads the ID code
        ' It assumes that the processor is at the proper state after reset.
        '


        ICSP_send_opcode(&H200FF0)
        ICSP_send_opcode(&H880190)
        ICSP_send_opcode(&H200006)
        ICSP_send_opcode(&H0)
        ICSP_send_opcode(&H0)
        ICSP_send_opcode(&H200007)
        ICSP_send_opcode(&HBA0BB6)
        ICSP_send_opcode(&H0)
        ICSP_send_opcode(&H0)
        ICSP_send_opcode(&H883C20)
        ICSP_send_opcode(&H0)


        x = ICSP_receive_word()
        ICSP_send_opcode(0)


        ICSP_send_opcode(&H200026)
        ICSP_send_opcode(&H0)
        ICSP_send_opcode(&H0)
        ICSP_send_opcode(&H200007)
        ICSP_send_opcode(&HBA0BB6)
        ICSP_send_opcode(&H0)
        ICSP_send_opcode(&H0)
        ICSP_send_opcode(&H883C20)
        ICSP_send_opcode(&H0)


        y = ICSP_receive_word()
        ICSP_send_opcode(0)

    End Sub
    Private Sub Button16_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button16.Click
        Dim x, y As Long

        ICSP_readID(x, y)

        Label3.Text = Hex(x) + " - " + Hex(y)

    End Sub

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        Dim x, y As Long

        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&H200000)      'MOV  #0x00,W0
        ICSP_send_opcode(&H880190)      'MOV  W0,TABLPAG
        ICSP_send_opcode(&H22BFC6)      'MOV  #0x2BFC,W6
        ICSP_send_opcode(&H207847)      'MOV  #VISI,W7
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&HBA0BB6)      'TBLRDL  [W6++],[W7]
        ICSP_send_opcode(&H0)           'NOP
        ICSP_send_opcode(&H0)           'NOP

        x = ICSP_receive_word()
        ICSP_send_opcode(0)

        ICSP_send_opcode(&HBA0BB6)      'TBLRDL  [W6++],[W7]
        ICSP_send_opcode(&H0)           'NOP
        ICSP_send_opcode(&H0)           'NOP

        y = ICSP_receive_word()
        ICSP_send_opcode(0)

        Label2.Text = "Config. Word: " + Hex(y) + " - " + Hex(x)

    End Sub

    Private Sub Button4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button4.Click

        ICSP_program_config_words()

        Label2.Text = "Config. Word Programmed!"

    End Sub

    Private Sub Button5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button5.Click

        ICSP_erase_flash_full()

        Label2.Text = "Memory and Config. Word Erased!"

    End Sub

    Private Sub Button6_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button6.Click
        Dim x, y As Long
        Dim i As Long

        ClearReadBuffer(instr)


        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&H200000)      'MOV  #0x00,W0
        ICSP_send_opcode(&H880190)      'MOV  W0,TBLPAG
        ICSP_send_opcode(&H200006)      'MOV  #0x0000,W6

        ICSP_send_opcode(&H207847)      'MOV  #VISI,W7
        ICSP_send_opcode(&H0)           'NOP


        For i = 1 To 5632
            If ((i - 1) Mod 64) = 0 Then
                Label4.Text = "reading row: " + Str((i - 1) \ 64) + " of 88"
                Application.DoEvents()

                ICSP_send_opcode(&H0)           'NOP
                ICSP_send_opcode(&H40200)       'GOTO 0x200
                ICSP_send_opcode(&H0)           'NOP

            End If
            ICSP_send_opcode(&HBA0B96)      'TBLRDL  [W6],[W7]
            ICSP_send_opcode(&H0)           'NOP
            ICSP_send_opcode(&H0)           'NOP
            x = ICSP_receive_word()
            'ICSP_send_opcode(&H0)           'NOP

            ICSP_send_opcode(&HBA8BB6)      'TBLRDH  [W6++],[W7]
            ICSP_send_opcode(&H0)           'NOP
            ICSP_send_opcode(&H0)           'NOP
            y = ICSP_receive_word()
            'ICSP_send_opcode(&H0)           'NOP

            instr(i) = (x And &HFFFF) Or ((y And &HFFFF) << 16)
        Next i



        ICSP_send_opcode(&H22B806)      'MOV  #0x2B80,W6
        ICSP_send_opcode(&H0)           'NOP (very critical: do NOT ommit !!!)


        For i = (&H2B80 / 2) + 1 To 5632
            ICSP_send_opcode(&HBA0B96)      'TBLRDL  [W6],[W7]
            ICSP_send_opcode(&H0)           'NOP
            ICSP_send_opcode(&H0)           'NOP
            x = ICSP_receive_word()

            ICSP_send_opcode(&HBA8BB6)      'TBLRDH  [W6++],[W7]
            ICSP_send_opcode(&H0)           'NOP
            ICSP_send_opcode(&H0)           'NOP
            y = ICSP_receive_word()

            instr(i) = (x And &HFFFF) Or ((y And &HFFFF) << 16)
        Next i




        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP


        WriteToFile(instr, "c:\pic24\MCUdump.txt")
        Label4.Text = "Dump complete!"

    End Sub
    Sub WriteToFile(ByRef x() As Long, ByVal nameout As String)
        '
        '   This subroutine writes to a text file the data held in 
        '   array x (assumed to have 5632 values).
        '
        Dim i, index As Long
        Dim d As Long
        Dim s, sa As String
        Dim fout As StreamWriter

        fout = New StreamWriter(nameout)


        For i = 1 To 5632
            index = (i - 1) * 2
            sa = Hex(index)
            If index < 16 Then sa = "0" + sa
            If index < 256 Then sa = "0" + sa
            If index < 4096 Then sa = "0" + sa
            sa = sa + ":"
            fout.Write(sa)


            d = x(i)
            s = Hex(d)
            If d < 16 Then s = "0" + s
            If d < 256 Then s = "0" + s
            If d < 4096 Then s = "0" + s
            If d < 65536 Then s = "0" + s
            If d < 1048576 Then s = "0" + s
            s = " " + s
            fout.Write(s)
            fout.WriteLine()
        Next i

        fout.Close()


    End Sub
    Function readfromfile(ByRef x() As Long) As Integer
        '
        '   This subroutine reads a text file with data 
        '   The data are held in array x (assumed to have 5632 values).
        '   Returns 0 if all O.K., 1 otherwise
        '   Assumes a global string variable "fnameIN" that keeps the path (and file) name.
        '   This variable is initaly set to the current directory (".").
        '
        Dim i As Integer
        Dim d As Integer
        Dim dx, factor As Long
        Dim index As Long
        Dim crc As Integer
        Dim fin As StreamReader
        Dim line As String
        Dim flag As Integer
        Dim bytecount As Integer
        Dim addressx, address, addresslo, addresshi As Long
        Dim recordtype As Integer
        Dim openFileDialog1 As New OpenFileDialog()



        openFileDialog1.InitialDirectory = fnameIN
        openFileDialog1.Filter = "hex files (*.hex)|*.hex|All files (*.*)|*.*"
        openFileDialog1.FilterIndex = 1
        openFileDialog1.RestoreDirectory = True

        If openFileDialog1.ShowDialog() = DialogResult.OK Then
            fnameIN = openFileDialog1.FileName
        Else
            Label3.Text = "Error: not a vald file selected"
            Return (1)
        End If


        flag = 0
        ClearReadBuffer(x)
        fin = New StreamReader(fnameIN)

        addresslo = 0
        addresshi = 0

        Do
            line = fin.ReadLine()

            If Mid(line, 1, 1) = ":" Then
                bytecount = Convert.ToInt32(Mid(line, 2, 2), 16)
                addressx = Convert.ToInt32(Mid(line, 4, 4), 16)
                recordtype = Convert.ToInt32(Mid(line, 8, 2), 16)
                crc = 0
                crc = (crc + bytecount) Mod 256
                crc = (crc + addressx Mod 256) Mod 256
                crc = (crc + addressx \ 256) Mod 256
                crc = (crc + recordtype) Mod 256

                If recordtype = 0 Then
                    addresslo = addressx
                    address = addresslo + (addresshi << 16)
                    factor = 1
                    dx = 0
                    For i = 1 To bytecount
                        d = Convert.ToInt32(Mid(line, i * 2 + 8, 2), 16)
                        dx = dx + factor * d
                        factor = factor * 256
                        crc = (crc + d) Mod 256

                        If (factor > &H1000000) Then
                            index = address \ 4 + 1
                            x(index) = dx
                            factor = 1
                            dx = 0
                            address = address + 4
                        End If
                    Next i
                ElseIf recordtype = 4 Then
                    d = Convert.ToInt32(Mid(line, 1 * 2 + 8, 2), 16)
                    crc = (crc + d) Mod 256
                    addresshi = 256 * d
                    d = Convert.ToInt32(Mid(line, 2 * 2 + 8, 2), 16)
                    crc = (crc + d) Mod 256
                    addresshi = addresshi + d
                ElseIf recordtype = 1 Then
                    flag = 2
                End If

                crc = (256 - crc) And &HFF

                If crc <> Convert.ToInt32(Mid(line, bytecount * 2 + 10, 2), 16) Then
                    flag = 1
                End If
            Else
                flag = 1
            End If
            '            line = fin.ReadLine()
        Loop Until (line Is Nothing) Or (flag <> 0)

        fin.Close()

        If flag = 1 Then
            ClearReadBuffer(x)
        End If
        If flag = 2 Then
            flag = 0
        End If
        Return (flag)

    End Function
    Sub ClearReadBuffer(ByRef ReadBuffer() As Long)
        '
        '   This subroutine clears the one-dimensional array that hols the data read
        '   from the MCU.
        '   It assumes 5632 instructions (for PIC24FJ16).
        '
        '
        Dim i As Long

        For i = 1 To 5632
            ReadBuffer(i) = &HFFFFFF
        Next i
    End Sub

    Private Sub Button13_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button13.Click
        Dim flag As Integer

        ClearReadBuffer(instr)
        flag = readfromfile(instr)
        If flag = 0 Then
            ProgramMCU(instr)
        End If
    End Sub
    Sub ProgramMCU(ByVal x() As Long)
        '
        'Assumes PIC24FJ16 (5632 words) = 88 blocks of 64 words each.
        '
        Dim flag As Integer
        Dim block As Integer
        Dim j As Integer
        Dim address As Long
        Dim index As Long
        Dim DestAddress As Long
        Dim datalo, datahi As Long
        Dim xx As Integer



        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&H24001A)      'MOV  #0x4001,W10
        ICSP_send_opcode(&H883B0A)      'MOV  W10,NVMCON



        address = 0
        For block = 1 To 88

            'check if all the 64-instruction block is FF
            flag = 0
            For j = 1 To 64
                index = (block - 1) * 64 + j
                If x(index) <> &HFFFFFF Then
                    flag = 1
                End If
            Next j

            If flag = 1 Then
                Label5.Text = "Write Block " & Str(block) & " of 88"
                Application.DoEvents()

                For j = 1 To 64
                    index = (block - 1) * 64 + j
                    address = (index - 1) * 2
                    DestAddress = address << 4
                    datalo = x(index) And &HFFFF
                    datahi = x(index) >> 16

                    ICSP_send_opcode(&H200000)      'MOV  #0x0000,W10
                    ICSP_send_opcode(&H880190)      'MOV  W0,TBLPAG
                    ICSP_send_opcode(&H200007 Or DestAddress)      'MOV  #0xdestaddres,W7

                    ICSP_send_opcode(&H200001 Or (datalo << 4))      'MOV  #0xdatalo,W1
                    ICSP_send_opcode(&H200002 Or (datahi << 4))      'MOV  #0xdatahi,W2

                    ICSP_send_opcode(&HBB0B81)      'TBLWTL  W1,[W7]
                    ICSP_send_opcode(&H0)           'NOP
                    ICSP_send_opcode(&H0)           'NOP

                    ICSP_send_opcode(&HBBCB82)      'TBLWTH.B  W2,[W7]
                    ICSP_send_opcode(&H0)           'NOP
                    ICSP_send_opcode(&H0)           'NOP

                Next j

                ICSP_send_opcode(&HA8E761)      'BSET  NVMCON, #WR (BIT 15)
                ICSP_send_opcode(&H0)           'NOP

                Do
                    ICSP_send_opcode(&H40200)       'GOTO 0x200
                    ICSP_send_opcode(&H0)           'NOP
                    ICSP_send_opcode(&H207841)      'MOV  #VISI,W1
                    ICSP_send_opcode(&H207602)      'MOV  NVMCON,W2
                    ICSP_send_opcode(&H780892)      'MOV  W2,[W1]
                    ICSP_send_opcode(&H0)           'NOP

                    xx = ICSP_receive_word()
                    ICSP_send_opcode(0)

                Loop Until (xx And &H8000) = 0


                ICSP_send_opcode(&H40200)       'GOTO 0x200
                ICSP_send_opcode(&H0)           'NOP

            Else
                Label5.Text = "Skip Block " & Str(block) & " of 88"
                Application.DoEvents()
            End If

        Next block

        Label5.Text = "- Programming MCU ended"

    End Sub

    Private Sub Button7_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button7.Click
        Dim x, y As Long

        Label6.Text = "--"
        ICSP_enable()
        ICSP_sync()
        ICSP_reset()

        ICSP_readID(x, y)
        Label6.Text = "ID: " + Hex(x) + " - " + Hex(y)

        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H40200)       'GOTO 0x200
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&H200000)      'MOV  #0x00,W0
        ICSP_send_opcode(&H880190)      'MOV  W0,TABLPAG
        ICSP_send_opcode(&H22BFC6)      'MOV  #0x2BFC,W6
        ICSP_send_opcode(&H207847)      'MOV  #VISI,W7
        ICSP_send_opcode(&H0)           'NOP

        ICSP_send_opcode(&HBA0BB6)      'TBLRDL  [W6++],[W7]
        ICSP_send_opcode(&H0)           'NOP
        ICSP_send_opcode(&H0)           'NOP

        x = ICSP_receive_word()
        ICSP_send_opcode(0)

        ICSP_send_opcode(&HBA0BB6)      'TBLRDL  [W6++],[W7]
        ICSP_send_opcode(&H0)           'NOP
        ICSP_send_opcode(&H0)           'NOP

        y = ICSP_receive_word()
        ICSP_send_opcode(0)

        Label6.Text = Label6.Text + ", Config. Word: " + Hex(y) + " - " + Hex(x)

        reset = 0
        ICSP_zero()

    End Sub


    Private Sub Button9_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button9.Click
        Dim x, y As Long
        Dim flag As Integer

        Label7.Text = "--"
        ICSP_enable()
        ICSP_sync()
        ICSP_reset()

        ICSP_readID(x, y)
        Label7.Text = "ID: " + Hex(x) + " - " + Hex(y)

        ICSP_erase_flash_full()
        Thread.Sleep(500)
        Label7.Text = "Memory and Config. Word Erased!"

        ICSP_program_config_words()
        Label7.Text = "Config. Word Programmed!"

        ClearReadBuffer(instr)
        flag = readfromfile(instr)
        If flag = 0 Then
            ProgramMCU(instr)
            Label7.Text = "- Programming MCU ended"
        End If

        reset = 0
        ICSP_zero()

    End Sub

    Private Sub Button10_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button10.Click

        ICSP_program_config_words_2()

        Label2.Text = "Config. Word Programmed!"

    End Sub
End Class
